import stan
import pickle
import pandas as pd
import numpy as np
import arviz as az
import matplotlib.pyplot as plt
import scipy.stats as scst
import seaborn as sb
import japanize_matplotlib

"""
           Excelファイルからの読み込み
"""
in_file_name = input('入力データファイル(*.xlsx) = ')
xlsx = pd.ExcelFile(in_file_name)
data_xlsx = pd.read_excel(xlsx)

ID = data_xlsx.values[:,0]
Y = data_xlsx.values[:,1]  
X = data_xlsx.values[:,2]  
N = len(Y)

for v in zip(ID, Y, X):
    print(v)

f = open('Results.txt', 'w')        #   出力用ファイル
mean_X = np.mean(X)
sd_X = np.std(X)
print(f'X:   mean = {mean_X:.2f}    sd = {sd_X:.3f}')
f.write(f'X:   mean = {mean_X:.2f}    sd = {sd_X:.3f}\n')
X = (X - mean_X) / sd_X

mean_Y = np.mean(Y)
sd_Y = np.std(Y)
print(f'Y:   mean = {mean_Y:.2f}    sd = {sd_Y:.3f}')
f.write(f'\nY:   mean = {mean_Y:.2f}    sd = {sd_Y:.3f}\n')
Y = (Y - mean_Y) / sd_Y

Data = {'N': N, 'Y': Y, 'X': X}
#
#          ビルドとサンプリング
#
with open('Regression.stan', 'r') as g:
    sm = stan.build(g.read(), data = Data)
fit = sm.sample()

d_frm = fit.to_frame()
i_data = az.from_pystan(posterior = fit, posterior_model = sm)

print(az.summary(i_data))
f.write('\n')
f.write(az.summary(i_data).__str__())

def med_mad_sd(x):  
    """   中央値とmad sdの計算
        Gelman et al. (2021), p.73
    """
    med = np.median(x)
    mad = np.median(np.abs(x - med))
    mad_sd = 1.483 * mad
    return med, mad_sd

med_b0, mad_sd_b0 = med_mad_sd(d_frm['b0'])
med_b1, mad_sd_b1 = med_mad_sd(d_frm['b1'])
med_sgm, mad_sd_sgm = med_mad_sd(d_frm['sgm'])
f.write('\n\n')
f.write(f'b0:   med.= {med_b0:.3f},   mad sd = {mad_sd_b0:.3f}\n')
f.write(f'\nb1:   med.= {med_b1:.3f},   mad sd = {mad_sd_b1:.3f}\n')
f.write(f'\nsgm:   med.= {med_sgm:.3f},   mad sd = {mad_sd_sgm:.3f}\n')

sb.kdeplot(d_frm['b0'])
plt.xlabel('b0', fontsize = 16)
plt.title(f'b0: med. = {med_b0:.3f},  mad sd = {mad_sd_b0:.3f}', fontsize = 16) 
plt.show()

sb.kdeplot(d_frm['b1'])
plt.xlabel('b1', fontsize = 16)
plt.title(f'b1: med. = {med_b1:.3f},  mad sd = {mad_sd_b1:.3f}', fontsize = 16) 
plt.show()

sb.kdeplot(d_frm['sgm'])
plt.xlabel('$\sigma$', fontsize = 14)
plt.title(f'$\sigma$: med. = {med_sgm:.3f},  mad sd = {mad_sd_sgm:.3f}',
          fontsize = 16) 
plt.show()

plt.plot(d_frm['b0'], d_frm['b1'], 'o', alpha = 0.3)
plt.xlabel('b0', fontsize = 16)
plt.ylabel('b1', fontsize = 16)
plt.tight_layout()
plt.show()
#
#          回帰直線と散布図
#
for i in range(len(ID)):
    plt.plot(X[i], Y[i], 'o', c='b')
    plt.text(X[i], Y[i], ID[i])
min_x = np.min(X)
max_x = np.max(X)
plt.plot([min_x, max_x], [med_b0 + med_b1*min_x, med_b0 + med_b1*max_x],
         label = f'Y = {med_b0:.3f} + {med_b1:.3f} * X')
plt.xlabel('X', fontsize = 14)
plt.ylabel('Y', fontsize = 14)
plt.legend()
plt.tight_layout()
plt.show()

f.close()
print('Results.txt was saved.')

